package bham.cs.fsc.exp;
import bham.cs.fsc.ui.*;
import bham.cs.fsc.gp.*;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.Button;
import java.awt.Event;
import java.awt.Label;
import java.awt.Component;
import java.awt.Color;

/**
* Adapted from original source by Hans U.Gerber for solving symbolic regression problem using GP.
* 
* This is the modal dialog for inspecting and editing some
* properties of the genetic algorithm.
*
* @version 1.0
* @author Phyllis Chong (<a href="mailto:fsc@cs.bham.ac.uk">fsc@cs.bham.ac.uk</a>)
*/
public class SettingsDialog extends Dialog
{
	private GP gp;
	private Button okButton;
	private Button cancelButton;
	private Label status;

	private void addNumberField(NumberUI param, int x, int y) {
		addComponent(new Label(param.getLabel(), Label.RIGHT), new Rectangle(
				//x - 143,
				x,
				y,
				//140,
				165,
				//14));
				20));
		addComponent(param, new Rectangle (
				x + 170,
				y,
				//30,
				35,
				//14));
				20));
		addComponent(new Label("(" + param.getMin() + " ... " + param.getMax() +")"),
				new Rectangle (
				x + 215,
				y,
				//55,
				65,
				//14));
				20));
	}

	private void addChoiceField(ChoiceUI param, int x, int y) {
		addComponent(new Label(param.getLabel(), Label.RIGHT), new Rectangle(
				x,
				//x - 143,
				y,
				140,
				//14));
				20));
		addComponent(param, new Rectangle (
				x + 145,
				y,
				90,
				//14));
				20));
	}

	/**
	*	Creates the dialog with all its components.
	* Note that the dialog uses Microsoft's DialogLayout manager.
	* All coordinates are measured in "Dialog Units", a measure
	* based on the font size.
	*/
	public SettingsDialog(Frame parent, GP gp, Problem p) {
	        //super(parent, new Rectangle(8, 8, 280, 282),
	        //super(parent, new Rectangle(8, 8, 295, 555),
	        super(parent, new Rectangle(8, 8, 295, 545),
				"GP Settings", true);
		//	Weird: The AWT is supposed to make the dialog window a child
		//	of the main frame. But it takes the desktop top left corner 
		//	as coordinate	origin instead of the frame's.
		//	On some systems, modal dialogs are fixed on the screen, therefore,
		//	I have to put my dialog near the top left corner, so that it is fully
		//	visible.
		this.gp = gp;
		//setResizable(false);
		setResizable(true);

		okButton = new Button("OK");
		addComponent(okButton, new Rectangle(
				5,
				//236,
				//535,
				//$513,
				502,
				50,
				14));
		cancelButton = new Button("Cancel");
		addComponent(cancelButton, new Rectangle(
				60,
				//236,
				//535,
				//$513,
				502,
				50,
				14));

		//addNumberField(gp.seed, 5, 3);
		addNumberField(gp.populationSize, 5, 3);
		addNumberField(gp.maxGenNum,5,25);
		addNumberField(gp.exportGenNum,5,47);
		addNumberField(gp.numToExport,5,69);
		addNumberField(GP.imigrantQueueSize,5,91);
		addNumberField(gp.maxDepthForNewIndividuals,5, 113);
		addNumberField(gp.crossoverFraction,5, 135);
		addNumberField(gp.fitnessProportionateReproFraction,5, 157);
		addNumberField(gp.mutationFraction,5, 179);
		addNumberField(gp.maxDepthForIndividualsAfterCrossover, 5, 201);
		addNumberField(gp.maxDepthForNewSubtreesInMutants, 5, 223);
		addNumberField(gp.tournamentSize, 5, 245);
		addChoiceField(gp.methodOfGeneration, 5, 267);
		addChoiceField(gp.methodOfSelection, 5, 289);
		
		addChoiceField(gp.elitismChosen, 5, 311);
		//addChoiceField(gp.methodOfSelection, 5, 311);

		//addNumberField(gp.maxDepthForNewIndividuals, 140, 18);
		//addNumberField(gp.crossoverFraction, 140, 33);
		//addNumberField(gp.fitnessProportionateReproFraction, 140, 48);
		//addNumberField(gp.mutationFraction, 140, 63);
		//addNumberField(gp.maxDepthForIndividualsAfterCrossover, 140, 78);
		//addNumberField(gp.maxDepthForNewSubtreesInMutants, 140, 93);
		//addChoiceField(gp.methodOfGeneration, 140, 108);
		//addChoiceField(gp.methodOfSelection, 140, 123);

		//addComponent(new Label(gp.fitnessCases.getLabel()), new Rectangle(5, 140, 50, 12));
		//addComponent(new Label(gp.fitnessCases.getLabel()), new Rectangle(15, 334, 50, 12));
		
		//addComponent(new Label(gp.fitnessCases.getLabel()), new Rectangle(15, 312, 50, 12));
		//addComponent(new Label(p.fitnessCases.getLabel()), new Rectangle(15, 312, 50, 12));

		//*addComponent(new Label(p.getFitnessCases().getLabel()), new Rectangle(15, 312, 50, 12));
		addComponent(new Label(p.getFitnessCases().getLabel()), new Rectangle(15, 334, 50, 12));

		/*
		addComponent(gp.fitnessCases, new Rectangle(
				5, 
				152, 
				98, 
				65));
				*/	
		/*
		addComponent(gp.fitnessCases, new Rectangle(
				5, 
				//346,
				324,
				280, 
				100));
				*/

		//addComponent(p.fitnessCases, new Rectangle(
		addComponent(((Component)p.getFitnessCases()), new Rectangle(
				5, 
				346,
				//324,
				280, 
				//100));
				60));

		/*
		addComponent(new Label(gp.functionSet.getLabel()), new Rectangle(110, 140, 50, 12));
		addComponent(gp.functionSet, new Rectangle(
				110, 
				152, 
				80, 
				65));
				*/

		//addComponent(new Label(gp.functionSet.getLabel()), new Rectangle(15, 452, 50, 15));
									       
									       
		//$addComponent(new Label(gp.functionSet.getLabel()), new Rectangle(15, 430, 50, 15));
		addComponent(new Label(gp.functionSet.getLabel()), new Rectangle(15, 410, 50, 15));

		addComponent(gp.functionSet, new Rectangle(
				5, 
				//467,
				//$445,
				425,
				80, 
				65));

		/*
		addComponent(new Label(gp.terminalSet.getLabel()), new Rectangle(195, 140, 50, 12));
		addComponent(gp.terminalSet, new Rectangle(
				195, 
				152, 
				80, 
				65));
				*/

		//addComponent(new Label(gp.terminalSet.getLabel()), new Rectangle(135, 452, 50, 15));
		//addComponent(new Label(gp.terminalSet.getLabel()), new Rectangle(135, 430, 50, 15));
		//addComponent(new Label(gp.terminalSet.getLabel()), new Rectangle(135, 420, 50, 15));
		addComponent(new Label(gp.terminalSet.getLabel()), new Rectangle(135, 410, 50, 15));
		addComponent(gp.terminalSet, new Rectangle(
				125, 
				//467,
				//445,
				//$435,
				425,
				80, 
				65));


		status = new Label();
		addComponent(status, new Rectangle(
				5,
				//222,
				//535,
				//530,
				525,
				280,
				10));
	}

	/**
	* Is called whenever the user clicks a command button.
	* On the "OK" button, the dialog validates all its fields.
	* If any field is found invalid, the error message returned
	* by the offending component is displayed.
	* If all fields are valid, their contents are accepted.
	*/
	public boolean action(Event evt, Object what) {
		if (evt.target == okButton) {
			String validateMsg = onApply();
			if (validateMsg == null) {
				status.setText("");
			} else {
				status.setText(validateMsg);
			}
			return true;
		}
		if (evt.target == cancelButton) {
			onCancel();
			return true;
		}
		return false;
	}

	public String onApply() {
		String result = super.onApply();
		if (result == null) {
			gp.settingsChanged();
		}
		return result;
	}

}








