package bham.cs.fsc.gp.ant;

import java.io.*;
import java.awt.*;
import bham.cs.fsc.gp.*;
import bham.cs.fsc.ui.*;
import bham.cs.fsc.exp.*;
//port gp.approx.ui.*;

/**
* Problem specific code to demonstrate the solving of the Artificial Ant problem.
*
* @version 1.0
* @author Phyllis Chong (<a href="mailto:fsc@cs.bham.ac.uk">fsc@cs.bham.ac.uk</a>)
*/

public abstract class Ant implements Problem {
//public class Ant {
  //public UPlot plot;
  public AntPlot plot;
  //public TrailUI fitnessCases;
  
  //public XYPlot plot; 
  //public PointsUI fitnessCases;

  public int NrOfFitnessCases = 1;
  static int nrOfAntImages = 4;

  public PlotUI fitnessCases;
  public String fitnessCaseString;

  public boolean maxHits = false;


  static int timeSteps;  
  static int foodEaten;
  static int X;
  static int Y;
  static int facing;
  
  public static boolean [][] food; //make public so that gp.ant.ui.AntPlot.java can access it.
  public static boolean [][] foodPlot; //make public so that gp.ant.ui.AntPlot.java can access it.

  static int numFoodPieces;
  public static int gridSize;  //make public so that gp.ant.ui.AntPlot.java can access it.  
  static String panelName;
  static int probTimeSteps;

  static final int NORTH = 1;
  static final int SOUTH = 2;
  static final int EAST = 3;
  static final int WEST = 4;

  //static boolean recAntPos = false;
  static RealPoint[] bestTrail;
  RealPoint[] bestTrailFound;
  static int trailArrayIndex;
  
  static GP gp;

  static DGPClient dgpClient; 

  //int previousHits = 0;

  /*
  public String getProblemName() {
    return "Artificial Ant"; 
  }
  */

  public Panel defPlotPanel(DGPClient dgpClient) {
    
    this.dgpClient = dgpClient;

     Panel plotPanel = new Panel();
     plotPanel.setLayout(new BorderLayout());
    //String probLabel = prob.getGraphicsLabel();
    //plotPanel.add("North", new Label(probLabel));
     
    plotPanel.add("North", new Label(panelName));
		
    plot = new AntPlot(dgpClient);
    //plot = new TrailPlot();
    //plot = prob.getGraphics();	
    plotPanel.add("Center", plot);
    
    return plotPanel;

  }
  
  public SetUI getTerminalSet() {

		ProgramChoice[] terminalChoices = {
			new ProgramChoice("Left", new Left().getClass()),
			new ProgramChoice("Right", new Right().getClass()),
			new ProgramChoice("Move", new Move().getClass())
		};
		int[] terminalSelections = {0, 1, 2};
		SetUI termSet = new SetUI("Terminal set", terminalChoices, terminalSelections);
		
		return termSet;
  }

  //public void fitnessFunction(Individual ind, GP gp, int individualNr) {
  public boolean fitnessFunction(Individual ind, GP gp, int individualNr) {
       /**
	* This function has to evaluate the fitness for a specific individual.
	* As a side-effect, it notfies any observers of the GP that an individual
	* has been evaluated and supplies the test cases.
	*/
        
	//*System.out.println("\nEvaluating individual " + individualNr);

           this.gp = gp;

	   //*System.out.println("inside fitnessFunction");
	   //*System.out.println("gp.evalBest : " + gp.evalBest);

           if (! gp.evalBest) {
		gp.notifyObservers(new GPMessageStatus("Evaluating individual #" + individualNr + "\n" +
					"of generation " + gp.currentGeneration));
	   }
	   /*
	   else {
	     
	     gp.notifyObservers(new GPMessageStatus("Displaying best individual of generation\n" +
					 gp.currentGeneration));

	   }
	   */
		boolean solutionFound = false;
                //double rawFitness = 0.0;
                //Ant.problemName = SantaFe;
                timeSteps = probTimeSteps;
                foodEaten = 0;
		//ind.hits = 0;
		X = 0;
		Y = 0;
		facing = Ant.EAST;
		placeFoodPieces();
		

		bestTrail = new RealPoint[probTimeSteps + 1];
		bestTrail[0] = new RealPoint(0.0,0.0);
		trailArrayIndex = 1;

		
		//evalFitness(ind, gp, individualNr, (double)gridSize);
		//**System.out.println("Evaluating program :\n" + ind.program.toString(0));
		//**System.out.println("Time steps is " + timeSteps + " Food eaten is " + foodEaten);		
                //while (timeSteps > 0) {
                while (! stopEval()) {
		  //System.out.println("Evaluating program ...");
		  //System.out.println("Time steps is " + timeSteps + " Food eaten is " + foodEaten);
		  //System.out.println("Food eaten is " + foodEaten);
		  //System.out.println("Max Hits is " + maxHits);

		  ind.program.eval(1.0);
		  //ind.program.eval(gridSize);

		  //**System.out.println("Finished evaluating one round.");
		  //**System.out.println("Time steps is " + timeSteps + " food eaten is " + foodEaten);
		  
		}

		//**System.out.println("Time steps is " + timeSteps + " stopEval() is " + stopEval() + " food eaten is " + foodEaten);
		//System.out.println("Food eaten is " + foodEaten);
		//System.out.println("Max Hits is " + maxHits);

		//ind.adjustedFitness = foodEaten;
		
		bestTrailFound = new RealPoint[trailArrayIndex];
		    
		for (int i = 0; i < trailArrayIndex; i ++) {
		  bestTrailFound[i] = bestTrail[i];
		}
		
		//*System.out.println("finished evaluating.");
		//*System.out.println("gp.evalBest : " + gp.evalBest);

		if ((! gp.evalBest) && (! gp.suddenEnd)) {
		  //*System.out.println("gp.evalBest is false : evaluating fitness and hits");
		  ind.standardizedFitness = numFoodPieces - foodEaten;
		  ind.hits = foodEaten;
		  
		  if (ind.hits == numFoodPieces) { 
		    maxHits = true; 
		    ind.sent = true;
		    solutionFound = true;
		  }
		  
		  if (trailArrayIndex > 1) {
		    //*System.out.println("drawing trace 1 for bestTrailFound");
		    gp.notifyObservers(new GPMessageEvaluatingIndividual(bestTrailFound));

		  }
		}
		
		return solutionFound;
		
  }
  

	public void normalizeFitnessOfPopulation(Individual[] population) {
		double sumOfAdjustedFitnesses = 0.0;
		double avgAdjFitness = 0.0;
		for (int i = 0; i < population.length; i++) {
			//	Set the adjusted fitness.
		  population[i].adjustedFitness = 
    			1.0 / (population[i].standardizedFitness + 1.0);
		  //population[i].adjustedFitness = 
		  //1.0 / (1.0 + (population[i].standardizedFitness / numFoodPieces));
			//	Add up the adjusted fitnesses so that we can normalize them.
			sumOfAdjustedFitnesses =
					sumOfAdjustedFitnesses + population[i].adjustedFitness;
		}
		//	Loop through population normalizing the adjusted fitness.
		for (int i = 0; i < population.length; i++) {
			population[i].normalizedFitness =
					population[i].adjustedFitness / sumOfAdjustedFitnesses;
		}
		avgAdjFitness = sumOfAdjustedFitnesses / population.length;
		System.out.println("\nAverage adjusted fitness of population for generation is : "  + avgAdjFitness);

	}
	
    
    public void evalBestOfRunIndividual(Individual bestIndv, GP gp) {
        /**
	* This function has to evaluate the fitness for a specific individual.
	* As a side-effect, it notfies any observers of the GP that an individual
	* has been evaluated and supplies the test cases.
	*/
     	
		  //this.gp = gp;

      System.out.println("Evaluating best of run individual ... ");
      
      //gp.evalBest = true;

      	gp.notifyObservers(new GPMessageBestFound(gp.currentGeneration,
						  bestIndv.program.toString(0),
						  //bestTrailFound,
						  null,
						  //bestIndv.adjustedFitness));
						  bestIndv.hits));
	
	String drawStatusTxt = "";

	if (dgpClient.drawChoice.getSelectedIndex() == 1) {
	  if (((ElitismSelection)gp.elitismChosen.getSelection()).getValue() == ElitismSelection.ELITISM_USED) {
	    drawStatusTxt = "Showing trail of best program in\nthe run";
	  }
	  else {
	    drawStatusTxt = "Showing trail of best program in\ngeneration " + gp.currentGeneration;
	  }
	}
	
	if (dgpClient.drawChoice.getSelectedIndex() == 0) {
	  if (gp.drawBest) {
	    if (((ElitismSelection)gp.elitismChosen.getSelection()).getValue() == ElitismSelection.ELITISM_USED) {
	      drawStatusTxt = "Showing trail of best program found\nin the run";
	    }
	    else {
	      drawStatusTxt = "Showing trail of best program found\nin the last generation";
	    }
	  }
	  else {
	    drawStatusTxt = "Showing trail of best program found\nin the last generation";
	  }
	}
       
	gp.notifyObservers(new GPMessageStatus(drawStatusTxt));

	
	/*
      gp.notifyObservers(new GPMessageResults("The best individual found\n" +
					"in generation " +
					gp.currentGeneration + "\n" +
					"has a hit count of " + bestIndv.hits + "\n\n" + 
					"Its function is:\n\n" + 
					bestIndv.program.toString(0)));
					*/
      /*
      bestTrail = new RealPoint[probTimeSteps + 1];
      bestTrail[0] = new RealPoint(0.0,0.0);
      trailArrayIndex = 1;
      */
      //bestTrail = new RealPoint[probTimeSteps];
      //trailArrayIndex = 0;
      //bestIndv.program.eval(1.0);
      
      //bestTrailFound = new RealPoint[trailArrayIndex];
      //System.arraycopy(bestTrail, 0, bestTrailFound, 0,trailArrayIndex);

	gp.evalBest = true;
	
	//*System.out.println("Calling fitnessFunction() to evaluate bestIndv");
	//*System.out.println("gp.evalBest : " + gp.evalBest);

      fitnessFunction(bestIndv, gp, 0);

      //*System.out.println("fitnessFunction completed. gp.evalBest : " + gp.evalBest);
      /*
      bestTrailFound = new RealPoint[trailArrayIndex];
		    
      for (int i = 0; i < trailArrayIndex; i ++) {
	bestTrailFound[i] = bestTrail[i];
      }


      System.out.println("trailArrayIndex : " + trailArrayIndex);
      System.out.println("best program : " + bestIndv.program.toString(0));

      for (int i = 0; i < trailArrayIndex ; i++) {
	System.out.println("bestTrail : " + bestTrail[i].x + "," + bestTrail[i].y);
	System.out.println("bestTrailFound : " + bestTrailFound[i].x + "," + bestTrailFound[i].y);
      }
      */

      gp.evalBest = false;
      //*System.out.println("setting gp.evalBest to false");
      //*System.out.println("gp.evalBest : " + gp.evalBest);

      if (trailArrayIndex > 1) {
	System.out.println("displaying bestFoundTrail in trace 2");

	gp.notifyObservers(new GPMessageBestFound(gp.currentGeneration,
						  bestIndv.program.toString(0),
						  bestTrailFound,
						  //null,
						  //bestIndv.adjustedFitness));
						  bestIndv.hits));
      }

      //recAntPos = false;
      //previousHits = 0;
	
  }
  
       /**
	* Computes the normalized and adjusted fitness of each
	* individual in the population.
	*/
        /*
	public void normalizeFitnessOfPopulation(Individual[] population) {
	  for (int i = 0; i < population.length; i++) {
			//	Set the adjusted fitness.
			population[i].standardizedFitness = numFoodPieces - population[i].adjustedFitness ; 
			population[i].normalizedFitness = population[i].adjustedFitness ;
			
			//*System.out.println("\npopulation[" + i + "].standardized fitness is " + population[i].standardizedFitness);
			//*System.out.println("population[" + i + "].normalized fitness is " + population[i].normalizedFitness + "\n");
		}
	}	
	*/       


   public double[] getFitnessesOfAllIndividuals(Individual[] population) {
     double[] rawFitness = new double[population.length];
     for (int i = 0; i < rawFitness.length; i++) {
       rawFitness[i] = numFoodPieces - population[i].standardizedFitness;
     }     
     return rawFitness;
   }


  //public abstract void makeDefaultFitnessCases(); 

  //abstract int getNewXCoordinate();
  
  //abstract int getNewYCoordinate();

  
  static int getNewXCoordinate(int newX) {
    if (newX > (int)gridSize - 1 ) {
      return 0;
    }
    else 
      if (newX < 0) {
      return (int)gridSize - 1;
    }
      else {
	return newX;
      }
  }
  
 static int getNewYCoordinate(int newY) {
    if (newY > (int)gridSize - 1 ) {
      return 0;
    }
    else 
      if (newY < 0) {
      return (int)gridSize - 1;
    }
      else {
	return newY;
      }
  }
  

 public void makeDefaultFitnessCases() {

    fitnessCaseString = "Santa Fe Trail (" + numFoodPieces + ") food pieces";
    fitnessCases = new TrailUI("Fitness cases", fitnessCaseString);
    
    food = new boolean[gridSize][gridSize];
    
    //initialise 2-D grid to false
    for (int i = 0; i <  gridSize; i++) {
      for (int j = 0; j < gridSize; j++) {
	food[i][j] = false;
      }
    }
    placeFoodPieces();

    // copy 'food' array into 'foodPlot' array to plot the ant trail at AntPlot.java. The 'food' array cannot be
    // used because it keeps changing as food is eaten up by the ant when evaluating each individual. 'foodPlot',
    // on the other hand, is static.
    
    foodPlot = new boolean[gridSize][gridSize];
    
    for (int i = 0; i <  gridSize; i++) {
      for (int j = 0; j < gridSize; j++) {
      foodPlot[i][j] = food[i][j];
      }
    }

    //System.arraycopy(food, 0, foodPlot, 0, gridSize);
    
 }
    

  public String getFitnessCaseString() {
    return fitnessCaseString;
  }

  public PlotUI getFitnessCases() {
    return fitnessCases;
  }


        /**
	* @returns <code>true</code> if the GP algorithm should terminate
	*/
        /*
	public boolean terminationPredicate() {
	  //return currentGeneration >= MAX_GENERATIONS;
	  //return currentGeneration >= maxGenNum.intValue();
	  if (currentGeneration >= maxGenNum.intValue() || maxHits) {
	    return true;
	  }
	}
	*/

        
        static boolean stopEval() {
	  if (foodEaten == numFoodPieces || timeSteps == 0) {
	    return true;
	  }
	  return false;
	}
	
 
  public Plot getPlot() {
    Plot pplot = plot;
    return pplot;
  }

  public int getNrOfFitnessCases() {
    return NrOfFitnessCases;
  }

  public void initMaxHits() {
    maxHits = false;
  }

  public boolean getMaxHits() {
    return maxHits;
  }

  public String getFitnessType() {
    return "a hit count";
  }

 
public Histogram createHistogram() {
  //System.out.println("Setting window for Santa Fe");
  //return new Histogram(0.0, 0.0, 1.0, 89.0); 
  return new Histogram(0.0, 0.0, 1.0, (double)numFoodPieces);
}


public FrameConstraint getPlotFrameConstraint() {
  return new FrameConstraint(
			     /*
			     0.0, 4,
			     0.0, 0,
			     0.5, -4,
			     0.1, 0
			     ) ;
			     */

			     0.0, 4,
			     0.0, 0,
			     0.5, -4,
			     0.5, 0
			     );
			     
}


public FrameConstraint getHistoFrameConstraint() {
  return new FrameConstraint(
			     /*
			     0.0, 4,
			     0.1, 0,
			     0.5, -4,
			     1.0, 0
			     ) ;
			     */

			     0.0, 4,
			     0.5, 0,
			     0.5, -4,
			     1.0, 0
			     ) ;

}

  public Image[] getImageArray() {
    
    return plot.getAntImages();
  }

  static boolean drawAnt() {
    if (dgpClient.drawChoice.getSelectedIndex() == 1) {
      return true;
    }
    if (dgpClient.drawChoice.getSelectedIndex() == 0) {
      if (gp.drawBest) {
	return true;
      } else {
	return false;
      }	
    }
    if (dgpClient.drawChoice.getSelectedIndex() == 2) {
      return false;
    }

      /*
    switch (dgpClient.drawChoice.getSelectedIndex()) {
    case 0 :
      return true;
      break;
    case 1 :
      if (gp.drawBest) {
	return true;
      } else {
	return false;
      }	
      break;
    case 2 :
      return false;
      break;
    }
    */

    return false;
  }    
  

  //public abstract static int getNewXCoordinate(int NewX);
  //public abstract static int getNewYCoordinate(int NewY);
  //public abstract static boolean stopEval();

  //public abstract String getFitnessCaseString();
  //public abstract PlotUI getFitnessCases();
 
  //public abstract String getProblemName();
  //public abstract Panel defPlotPanel();
  public abstract String getProblemName();
  public abstract SetUI getFunctionSet();
  //public abstract void fitnessFunction(Individual ind, GP gp, int IndividualNr);
  //public abstract void normalizeFitnessOfPopulation(Individual[] population);
  //public abstract void makeDefaultFitnessCases();
  abstract void placeFoodPieces();
  //public abstract void setPWindow();
  //public abstract Histogram createHistogram();

}















