package bham.cs.fsc.gp.ant;

import java.awt.Graphics;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.FontMetrics;
import java.awt.Point;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.io.Serializable;
import java.math.*;
import java.net.URL;
import java.net.MalformedURLException;
import bham.cs.fsc.ui.*;
import bham.cs.fsc.exp.*;


/**
*
*	A class for plotting the ant trail, showing the path taken by the ant.
*
* @version 1.1
* @author Phyllis Chong <a href="mailto:fsc@cs.bham.ac.uk">fsc@cs.bham.ac.uk</a>)
*/

public class AntPlot extends Plot implements Serializable {
        PlotData[][] trace;
	Dimension previousSize = null;

        static final int FRAME_WIDTH = 4;
	int textWidth = 0;
	int textAscent = 0;

        int which;
        private static final Point origin = new Point(4,4);
  
        float cellWidthF;
        float cellHeightF;

        int cellWidthI;
        int cellHeightI;

        //boolean drawAnt;

        //Dimension d;
  
        Dimension offDimension;
        Image offImage;
        Graphics offGraphics;

        Image[] antImages;

        DGPClient dgpClient;

 
        public AntPlot(DGPClient dgpClient) {

	        this.dgpClient = dgpClient;
	  
	        //System.out.println("inside AntPlot constructor()");
	        //trace = new RealPoint[3][];
	        trace = new PlotData[3][];

		// Cannot copy Ant.food array into foodPlot array here because the 'placeFoodPieces' method is not called 
		// until later and Ant.food array would be null.
		// The non-changing foodPlot array is created at the 'placeFoodPieces' method of Ant.java instead.

		/*
		if (trace == null) {System.out.println("trace array is null ...");}
		else {System.out.println("trace array is not null ...");}
		//setWindow(0.0, -1.1, 1.05, 1.1);
		*/
	}
  
        /*
	void setViewport() {
		Dimension d = size();
		super.setViewport(4 + textWidth + 3 + 3, 4, d.width - 4, d.height - 4);
	}
	*/


        void initOffScreen(Dimension d) {
	  if ((offGraphics == null) || (d.width != offDimension.width) || (d.height != offDimension.height)) {
	    offDimension = d;
	    offImage = createImage(d.width, d.height);
	    offGraphics = offImage.getGraphics();
	  }
	}

       Image[] getAntImages() {

	 antImages = new Image[Ant.nrOfAntImages];

	 Toolkit toolkit = this.getToolkit();
	 String imageSrcURL;
	 
	 if (dgpClient.Application) {
	   imageSrcURL = "http://studentweb.cs.bham.ac.uk/~fsc/DGP/";
	 }
	 else {
	   imageSrcURL = dgpClient.getCodeBase().toString();
	   //System.out.println("imageSrcURL : " + imageSrcURL);
	   
	 }
	 try {
	   for (int i = 0; i < Ant.nrOfAntImages; i++) {
	     String imageURL = imageSrcURL + "AntN" + i + ".gif";
	     //System.out.println("imageURL : " + imageURL);
	     //antImages[i] = toolkit.getImage(new URL(imageSrcURL + "AntS" + i + ".gif"));
	     antImages[i] = toolkit.getImage(new URL(imageURL));
	     /*
	     if (antImages[i] == null) {
	       //System.out.println("antImages[" + i + "] is null !");
	     }else {//System.out.println("antImages[" + i + "] is not null !");
	     }
	     */
	   }
	 }
	 catch (MalformedURLException e) {System.out.println("ImageSrcURL malformed !");}

	 return antImages;
       }


        //public synchronized void setTrace(int which, RealPoint[] p) {
	public synchronized void setTrace(int which, PlotData[] p) {  
		//	I want to update a new trace as quickly and as smoothly as I can.
		//	I don't use double-buffering because it does not make sense to
		//	move whole bitmaps worth kilobytes when just a few pixels have to
		//	be updated.
		//	Requesting repaint() does not work with me because
		//	- it "calls update() as soon as possible" and several repaint
		//	  requests may be merged into one
		//	-	by calling update(), the background would be erased
		//	getGraphics() should work fine for this task but then I ran into
		//	a JDK AWT bug. It showed up with appletviewer and HotJava:
		//	I should be able to getGraphics() as many times as I want.
		//	But it seems to me that Windows resources (DCs?) are not properly
		//	freed. After a lot of calls, getGraphics() returns null.
		//	MS' jview and Explorer 3.0 ran fine, however.
		//	When I replaced MS' implementation of Win32Graphics.class in Sun's
		//	classes.zip, the problem went away. As a precaution, I now explicitely
		//	dispose() the graphics context.
		//	But then, this happened: 
		//	When testing the applet with Sun's JDK java interpreter and 
		//	appletviewer under Windows 95, the plot did not appear initially.
		//	I noticed that setTrace() caused paint() to be called when the
		//	size of the canvas was still [0,0]. Maybe the AWT then assumed that
		//	the canvas was already painted and did not repaint again.
		//	Now I have setTrace() call paint() only if the size is not zero:
		//	
	        this.which = which;
		//System.out.println("which of setTrace is : " + which);
		Graphics g = getGraphics();
		Dimension d = size();
	
		//initOffScreen(d);
		
		if ((dgpClient.tracker.statusAll(false) & MediaTracker.ERRORED) != 0) {
		  System.out.println("Image downloading not completed/errored at setTrace()!");
		  g.clearRect(0, 0, d.width, d.height);
		  g.drawString("Please wait ...", d.width/4, d.height/2);
		}
	        else  
		  if (dgpClient.tracker.statusAll(false) == MediaTracker.COMPLETE) {		  		  
		    
		    //System.out.println("Image downloading completed. Going to draw all traces!");

		    boolean canPaint = (g != null && d.width != 0 && d.height != 0);
		    /*
		    if (canPaint) {
		      //	Erase the current image of the trace by painting it with
		      //	the background color. This may cut holes into other traces
		      //	and the axis. Therefore, I will repaint the whole plot after
		      //	this:
		      //setViewport();
		      eraseAnt = true;
		      */
		      /*
			if (trace[which] == null) {
			System.out.println("current trace[" + which + "] is null. ");
			} else {
			System.out.println("current trace[" + which + "] is not null. ");
			}
		      */
		      /*
		      if (which != 0) {
			System.out.println("erasing trace["+ which +"] ... eraseAnt set to true");
			drawTrace(g, trace[which], getBackground());
		      }
		    }
		    */

		    trace[which] = p;
		    
		/*
		  System.out.println("assigning new values to trace[" + which + "]");
		  if (trace[which] == null) {System.out.println("newly assigned trace is null !");}
		  else {System.out.println("newly assigned trace is not null !");}
		*/
		    
		    if (canPaint) {
		      //	Repaint all to fix any holes:
		      //drawAnt = true;
		      
		      //System.out.println("repainting everything. including the 3 traces ... drawAnt set to true");
			paint(g);
		      
		      //System.out.println("back from paint. Going to drawImage in buffer !");
		      /*
			boolean drawn = g.drawImage(offImage, 0,0, this);
			if (drawn) {System.out.println("Image drawn !");}
			else {System.out.println("Image drawing not completed !");}
		      */
		    }
		  }

		    if (g != null) {
		      //	As a quick fix for Sun's Windows resource leak bug,
		      //	I explicitely free the graphics context:
		      
		      //System.out.println("graphics object g disposed.");
			g.dispose();
			//	(Before  knew about dispose(), I used to do this:
			//	Collect garbage from time to time, hoping that
			//	Windows resources are freed as well:
			//
			//	counter++;
			//	if (counter > 100) {
			//		System.gc();
			//		counter = 0;
			//	}
		    }
		  }
	
	
  public synchronized void update(Graphics g) {
    //System.out.println("in update() !!!");
    paint(g);
  }
	
	/**
	* This method repaints the plot completely.
	*/
	public synchronized void paint(Graphics g) {
		//	After resizing a component, the AWT doesn't seem to call neither
		//	repaint() nor update(). It goes directly at paint().
		//	But I want a cleanly erased canvas after a resize, because not all
		//	of my drawing takes place in repaint().
		//	I force an update() (and therefore erasing the canvas)
		//	whenever paint() notices that the size of the canvas has changed:
		//
	        Dimension d = size();

		//System.out.println("offScreen object initialised.");
		initOffScreen(d);

		if (previousSize == null || previousSize.width != d.width || previousSize.height != d.height) {
			previousSize = d;
			update(g);
			//	This will erase the background and then call paint() recursively.
			//	Recursion stops as soon as changing the size stops.
		}


		if (textWidth == 0) {
			FontMetrics metrics = g.getFontMetrics();
			textWidth = metrics.stringWidth("+1");
			textAscent = metrics.getAscent();
		}

		//	Draw a 3-D frame:
		
		
		//System.out.println("Drawing 3-D frame");
		

		Color bg = getBackground();

		//System.out.println("Background colour is " + bg.toString());

		if (bg != Color.lightGray) {
		  bg = Color.lightGray;
		
		  //System.out.println("Background colour changed to " + bg.toString());
		}
		  
		/*
		g.setColor(bg);
		g.draw3DRect(0, 0, d.width - 1, d.height - 1, true);
		g.draw3DRect(3, 3, d.width - 7,	d.height - 7, false);
		g.setColor(Color.black);
		g.clipRect(4, 4, d.width - 8, d.height - 8);	
		*/

		offGraphics.setColor(bg);
                 
		offGraphics.draw3DRect(0, 0, d.width - 1, d.height - 1, true);
		offGraphics.draw3DRect(3, 3, d.width - 7,	d.height - 7, false);
		offGraphics.setColor(Color.black);
		offGraphics.clipRect(4, 4, d.width - 7, d.height - 7);
		offGraphics.setColor(bg);
		offGraphics.fillRect(4, 4, d.width - 7, d.height - 7);
		

		//setViewport();
		
		//**System.out.println("Width of Square : " + size().width);
		//**System.out.println("Height of Square : " + size().height);

		/*
		double cellWidthD = (size().width - origin.x) /(double)Ant.gridSize;
		double cellHeightD = (size().height - origin.y) /(double)Ant.gridSize;
		*/		
		cellWidthF = (size().width - origin.x) /(float)Ant.gridSize;
		cellHeightF = (size().height - origin.y) /(float)Ant.gridSize;
				
		//**System.out.println("cell Width (float) : " + cellWidthF);
		//**System.out.println("cell Height (float) : " + cellHeightF);

		/*
		cellWidth = (new BigDecimal(cellWidthDouble)).ROUND_HALF_DOWN; 
		cellHeight = (new BigDecimal(cellHeightDouble)).ROUND_HALF_DOWN;
		*/
		/*
		cellWidth = (int)((size().width - origin.x) / (double)Ant.gridSize);
		cellHeight = (int)((size().height - origin.y) / (double)Ant.gridSize);
		
		System.out.println("cell Width (int) : " + cellWidth);
		System.out.println("cell Height (int) : " + cellHeight);
		*/
		/*
		int roundCellWidth = Math.round(cellWidthL);
		int roundCellHeight = Math.round(cellHeightL);
		//int roundCellHeight = new BigDecimal(cellHeightD).ROUND_HALF_DOWN;

		System.out.println("cell Width (round) : " + roundCellWidth);
		System.out.println("cell Height (round) : " + roundCellHeight);
		*/ 

		cellWidthI = (int)(cellWidthF -1);
		cellHeightI = (int)(cellHeightF - 1);
		
		//System.out.println("drawing empty grid ...");
		
		//g.setColor(Color.white);
		offGraphics.setColor(Color.white);
		
		for (int i = 0; i < Ant.gridSize; i++) {
		  for (int j = 0; j < Ant.gridSize; j++) {
		    //g.fillRect(
		    offGraphics.fillRect(	       
			       Math.round(origin.x + i*cellWidthF),
			       Math.round(origin.y + j*cellHeightF), 
			       cellWidthI, 
			       cellHeightI
			      );
		  }
		}
		

		/*		
		drawTrace(g, trace[0], new Color(255, 0, 0)); //red
		drawTrace(g, trace[1], new Color(0, 0, 0));   //black
		drawTrace(g, trace[2], new Color(0, 0, 255)); //blue 
		*/
		
		
		//which = 0;
		drawTrace(offGraphics, trace[0], Color.yellow, 0); //gray
		//drawTrace(g, trace[0], Color.green); //green
		
		//which = 1;
		//drawTrace(offGraphics, trace[1], Color.red, 1);   //black
		drawTrace(offGraphics, trace[1], new Color(255, 0, 0), 1);   //red
		//drawTrace(g, trace[1], new Color(255, 0, 0));   //red
		
		//which = 2;
		drawTrace(offGraphics, trace[2], Color.black, 2);	//black
		//drawTrace(offGraphics, trace[2], new Color(0, 0, 255));	//blue
		//drawTrace(g, trace[2], new Color(0, 0, 255));	//blue   

		boolean drawn = g.drawImage(offImage, 0,0, this);
		/*
		if (drawn) {System.out.println("buffer Image drawn !");}
		else {System.out.println(" buffer Image drawing not completed !");}	
		*/
	}


	/**
	* Draws one of the 3 traces.
	*/
        //synchronized void drawTrace(Graphics g, int which, RealPoint[] trace, Color color) {
        synchronized void drawTrace(Graphics c, PlotData[] trace, Color color, int traceNum) {
		if (trace != null) {

		  //System.out.println("Trace[" + which + "] not null");

			int nrOfPoints = trace.length;
			
			//System.out.println("Trace has " + nrOfPoints + " items");
			
			c.setColor(color);
			//System.out.println("Graphics context colour set to " + color.toString());
			//System.out.println("Going to select which trace to run ... ");

			switch (traceNum) {
			case 0 :
			  Color cellColor = color;
			  
			  //System.out.println("drawing trace[0]...");
			  //System.out.println("color used is " + color.toString());
			  
			  for (int i = 0; i < nrOfPoints; i++) {

			    if (Ant.foodPlot[(int)trace[i].x][(int)trace[i].y]) {
			      cellColor = Color.orange;
			      //c.setColor(Color.orange);
			      c.setColor(cellColor);
			    }
			    else {
			      cellColor = color; 
			      //c.setColor(color);
			      c.setColor(cellColor);
			      //System.out.println("point[" + trace[i].x + "," + trace[i].y + "] 's color is set to " + color.toString());
			    }
			    
			    //System.out.println("Filling rectangle " + trace[i].x + "," + trace[i].y + " with color " + rectColor.toString());
			    c.fillRect(
				       Math.round((float)(origin.x + trace[i].x*cellWidthF)),
				       Math.round((float)(origin.y + trace[i].y*cellHeightF)),
				       cellWidthI, 
				       cellHeightI
				       );
			  }

			  if (which == 0 ) {
			    //c.setColor(Color.black);
			    //c.drawImage(antImages[0],origin.x, origin.y, cellWidthI, cellHeightI, cellColor, this);
			    //c.drawImage(antImages[2],origin.x, origin.y, cellWidthI, cellHeightI, cellColor, this);
			    c.drawImage(antImages[2],origin.x, origin.y, cellWidthI, cellHeightI, this);
			    //c.drawImage(antImages[0],origin.x, origin.y,cellColor, this);
			    //c.setColor(cellColor);
 
			    //System.out.println("AntImage[0] drawn at coordinates : " + origin.x + "," + origin.y + " with width & height : " + cellWidthI + "," + cellHeightI);  
			  }

			    /*
			    try {
			      wait(20);
			      System.out.println("waiting for 20s");
			    }
			    catch (InterruptedException e) {}			      
			    */

			    //}
			  //for (int i = 0; i < Ant.gridSize; i++) {
			  //for (int j = 0; j < Ant.gridSize; j++) {
			  //  if (Ant.food[i][j]) {
			    /*
			    if (Ant.food[(int)trace[i].x][(int)trace[i].y]) {
			      g.setColor(Color.black);
			      g.fillOval(
					 Math.round((float)(origin.x + trace[i].x*cellWidthF)),
					 Math.round((float)(origin.y + trace[i].y*cellHeightF)), 
					 //Math.round((float)(origin.x + i*cellWidthF)),
					 //Math.round((float)(origin.y + j*cellHeightF)), 
					 cellWidthI, 
					 cellHeightI
					 );
			      g.setColor(color);
			      }
			      */

			  //}
			  //System.out.println("trace[0] drawn !");
			  break;


			  //g.drawLine(x[i], y[i], x[i+1], y[i+1]);

			case 1 :
			  
			  //*System.out.println("drawing trace " + traceNum );
			  //System.out.println("color used is " + color.toString());
			  
			  //*System.out.println("evalBest : " + Ant.gp.evalBest);

			  //if ((Ant.recAntPos) && (traceNum == 1)) {
			  if (Ant.gp.evalBest) {
			    Image AntImage = antImages[0];

			    switch (Ant.facing) {
			    case 1 : 
			      AntImage = antImages[0];
			      //System.out.println("Ant image selected is 0");
			      break;
			    case 2 :
			      AntImage = antImages[1];
			      //System.out.println("Ant image selected is 1");
			      break;
			    case 3 :
			      AntImage = antImages[2];
			      //System.out.println("Ant image selected is 2");
			      break;
			    case 4 :
			      AntImage = antImages[3];
			      //System.out.println("Ant image selected is 3");
			      break;
			    }
			    

			  //if (drawAnt) {
			  //for (int i = 0; i < nrOfPoints - 1; i++) {

			  for (int i = 0; i < nrOfPoints; i++) {
			    /*
			      if (AntImage == null) {
				System.out.println("AntImage selected is null !");
			      }
			      else {
				System.out.println("AntImage selected is not null !");
				*/
			    if (AntImage != null) {
				//c.setColor(Color.black);
				//System.out.println("Image color set to " + Color.black.toString());
				c.drawImage(
					  AntImage,
					  Math.round((float)(origin.x + trace[i].x*cellWidthF)),
					  Math.round((float)(origin.y + trace[i].y*cellHeightF)),  
					  cellWidthI-2, 
					  cellHeightI-2, 
					  //Color.black,
					  this
					  );
				//System.out.println("AntImage drawn at " + Math.round((float)(origin.x + trace[i].x*cellWidthF)) + "," + Math.round((float)(origin.y + trace[i].y*cellHeightF)));
			      }
			    }
			  c.setColor(Color.black);

			    if (Ant.trailArrayIndex > 1) {
			      //System.out.println("Drawing line from trailArray");
			      for (int h = 0; h < Ant.trailArrayIndex - 1; h++) {
				//to prevent the drawing of line when ant wraps around to the other side
				if (toDrawLine(Ant.bestTrail[h], Ant.bestTrail[h+1])) {
				  
				  c.drawLine(
					     Math.round((float)(origin.x + Ant.bestTrail[h].x*cellWidthF)) + (int)((cellWidthI/3)*2), 
					     Math.round((float)(origin.y + Ant.bestTrail[h].y*cellHeightF)) + (int)((cellHeightI/3)*2), 
					     Math.round((float)(origin.x + Ant.bestTrail[h+1].x*cellWidthF)) + (int)((cellWidthI/3)*2),
					     Math.round((float)(origin.y + Ant.bestTrail[h+1].y*cellHeightF)) + (int)((cellHeightI/3)*2)
					     );
				}
				eatFood(Ant.bestTrail[h], c, Color.blue, Color.black);
			      }
			      eatFood(Ant.bestTrail[Ant.trailArrayIndex - 1], c, Color.blue, Color.black);
			    }
			  }
			  else {
			    for (int h = 0; h < nrOfPoints -1 ; h++) {
			      if (toDrawLine(trace[h], trace[h+1])) {
				c.drawLine(
				       Math.round((float)(origin.x + trace[h].x*cellWidthF)) + (int)(cellWidthI/3), 
				       Math.round((float)(origin.y + trace[h].y*cellHeightF)) + (int)(cellHeightI/3), 
				       Math.round((float)(origin.x + trace[h+1].x*cellWidthF)) + (int)(cellWidthI/3),
				       Math.round((float)(origin.y + trace[h+1].y*cellHeightF)) + (int)(cellHeightI/3)
				       );
			      }
			      eatFood(trace[h], c, color, color);
			    }
			    eatFood(trace[nrOfPoints - 1], c, color, color);
			  }

			  //System.out.println("trace " + traceNum + " drawn !");    
			  break;
			  
			case 2 :

			  for (int h = 0; h < nrOfPoints -1 ; h++) {
			    if (toDrawLine(trace[h], trace[h+1])) {
			      c.drawLine(
					 Math.round((float)(origin.x + trace[h].x*cellWidthF)) + (int)((cellWidthI/3)*2), 
					 Math.round((float)(origin.y + trace[h].y*cellHeightF)) + (int)((cellHeightI/3)*2), 
					 Math.round((float)(origin.x + trace[h+1].x*cellWidthF)) + (int)((cellWidthI/3)*2),
					 Math.round((float)(origin.y + trace[h+1].y*cellHeightF)) + (int)((cellHeightI/3)*2)
					 );
			    }

			    eatFood(trace[h], c, Color.blue, color);
			  }
			  eatFood(trace[nrOfPoints -1], c, Color.blue, color);
			  
			  //System.out.println("trace " + traceNum + " drawn !");			  
			  break;
			  

			  /*      
			 case 2 :
			      
			  System.out.println("drawing trace[2]...");
			  System.out.println("color used is " + color.toString());
  
			  for (int h = 0; h < nrOfPoints -1 ; h++) {
			  */
			    /*
			    c.drawOval(
				       //Math.round((float)(origin.x + trace[i].x*cellWidthF))-1,
				       //Math.round((float)(origin.y + trace[i].y*cellHeightF))-1, 
				       //(origin.y + ((int)trace[i].y)*cellHeight)-1, 
				       //cellWidthI + 2, 
				       //cellHeightI + 2

				       Math.round((float)(origin.x + trace[i].x*cellWidthF))+1,
				       Math.round((float)(origin.y + trace[i].y*cellHeightF))+1, 
				       cellWidthI - 2, 
				       cellHeightI -2

				       );
				       */
			 
			    //if (Ant.trailArrayIndex > 1) {
			    //System.out.println("Drawing line from trailArray");
			    //for (int h = 0; h < Ant.trailArrayIndex - 1; h++) { 
			  /*
			    c.drawLine(
				       Math.round((float)(origin.x + Ant.bestTrail[h].x*cellWidthF)) + (int)(cellWidthI/3), 
				       Math.round((float)(origin.y + Ant.bestTrail[h].y*cellHeightF)) + (int)(cellHeightI/3), 
				       Math.round((float)(origin.x + Ant.bestTrail[h+1].x*cellWidthF)) + (int)(cellWidthI/3),
				       Math.round((float)(origin.y + Ant.bestTrail[h+1].y*cellHeightF)) + (int)(cellHeightI/3)
				       );
			  } 

			  
			  System.out.println("trace[2] drawn !");
			  break;
			  */
			}
		}
	}

  boolean toDrawLine(PlotData point1, PlotData point2) {
    if ((Math.abs(point1.x - point2.x) == (Ant.gridSize - 1)) || (Math.abs(point1.y - point2.y) == (Ant.gridSize - 1))) {
      return false; 
    }
    return true ;
  }

  void eatFood(PlotData point1, Graphics c, Color fcolor, Color bcolor) {
    //System.out.println("inside eatFood(). setting color to blue");
    //c.setColor(Color.blue);
    c.setColor(fcolor);
    if (Ant.foodPlot[(int)point1.x][(int)point1.y]) {
      c.fillOval(
		 Math.round((float)(origin.x + point1.x*cellWidthF))+1,
		 Math.round((float)(origin.y + point1.y*cellHeightF))+1, 
		 cellWidthI - 2, 
		 cellHeightI -2	 
		 );
      //System.out.println("filling oval at " + Math.round((float)(origin.x + point1.x*cellWidthF))+1 + "," + Math.round((float)(origin.y + point1.y*cellHeightF))+1);
    }
    c.setColor(bcolor);
  }

}
