package bham.cs.fsc.gp.ant;

import java.io.*;
import java.awt.TextArea;
import java.awt.Font;
import bham.cs.fsc.ui.*;

/**
* Modified from original program by Hans U. Gerber for the artificial ant problem.
*
* This class combines the data model and the view to inspect and
* edit a series of x-y value pairs.
* The model stores both a raw text representation and the same data
* converted into an array of points.
* The raw text is stored because the user may embed comments into
* his input. These comments would get lost if only the numeric point
* data was stored.
*
* @version 1.0
* @author Phyllis Chong (<a href="mailto:fsc@cs.bham.ac.uk">fsc@cs.bham.ac.uk</a>)
*/
//public class PointsUI extends TextArea implements Validatable {
//public class TrailUI extends TextArea implements Validatable {

public class TrailUI extends TextArea implements PlotUI {
//public class TrailUI extends TextArea implements Validatable, PlotUI {

	private String label;
        //private RealPoint[] data;
        private PlotData[] data;
        //private String[] data;
	private String rawText;

        //public PointsUI(String label, String rawText) {
        public TrailUI(String label, String rawText) {
		super();
		setFont(new Font("Courier", Font.PLAIN, 10));
		this.label = label;
		this.rawText = rawText;
		//this.data = null;
		this.data = defTrail();
		display();
	}

	public String getLabel() { return label; }

	public void display() {
		setText(rawText);
	}

        /*
	public RealPoint[] get() {
		return data;
	}
	*/

        public PlotData[] get() {
		return data;
	}


	/**
	* Checks if the contents of the edit field are valid,
	* i.e. the user entered at least 10 x-y pairs.
	* @return <code>null</code> if valid, an error message
	*		otherwise.
	*/
        /*
	public String check() {
		RealPoint[] temp = read(getText());
		boolean ok = (temp != null && temp.length >= 10);
		if (ok) {
			return null;
		} else {
			return "\"" + label + "\" must contain at least 10 x-y-value pairs.";
		}
	}
	*/

        public String check() { return null;}

	/**
	* Copies the raw text and the x-y values (read from the same raw text)
	* from the edit field into the model. This method assumes that the
	* contents of the field have already been validated.
	*/
        /*
	public void accept() {
		rawText = getText();
		data = read(rawText);
	}
	*/
  
        public void accept() {;}

        //static final int MAX_NR_OF_FITNESS_CASES = 100;

	/**
	* Decodes the contents of the edit field into a series of x-y value
	* pairs.<br>
	* Note that the strean tokenizer used here does not understand numbers
	* with an exponent (like 1.23E+3).
	* @return an array of x-y points
	*/
  /*
	private RealPoint[] read(String s) {
		StringBufferInputStream stream = new StringBufferInputStream(s);
		StreamTokenizer tokenizer = new StreamTokenizer(stream);
		RealPoint[] temp = new RealPoint[MAX_NR_OF_FITNESS_CASES];
		int count = 0;
		try {
			int token;
			do {
				double x;
				double y;
				token = tokenizer.nextToken();
				if (token == StreamTokenizer.TT_NUMBER) {
					x = tokenizer.nval;
					token =	tokenizer.nextToken();
					if (token	== StreamTokenizer.TT_NUMBER) {
						y = tokenizer.nval;
						temp[count] = new RealPoint(x, y);
						count++;
					}
				}
			} while (count < MAX_NR_OF_FITNESS_CASES && token != StreamTokenizer.TT_EOF);
		} catch (Exception e) {
			;
		}
		RealPoint[] result = new RealPoint[count];
		System.arraycopy(temp, 0, result, 0, count);
		return result;
	}
	*/
  
  /* although package or private is sufficient for read(String s) method, it is declared as public in order to 
     implement the corresponding method in ui.PlotUI --- a limitation of Java. */

        public PlotData[] defTrail() {
	  
	  PlotData[] trail = new RealPoint[145];
	  trail[0] = new RealPoint(0.0, 0.0);
	  trail[1] = new RealPoint(1.0, 0.0);
	  trail[2] = new RealPoint(2.0, 0.0);
	  trail[3] = new RealPoint(3.0, 0.0);
	  trail[4] = new RealPoint(3.0, 1.0);
	  trail[5] = new RealPoint(3.0, 2.0);
	  trail[6] = new RealPoint(3.0, 3.0);
	  trail[7] = new RealPoint(3.0, 4.0);
	  trail[8] = new RealPoint(3.0, 5.0);
	  trail[9] = new RealPoint(4.0, 5.0);
	  trail[10] = new RealPoint(5.0, 5.0);
	  trail[11] = new RealPoint(6.0, 5.0);
	  trail[12] = new RealPoint(7.0, 5.0);
	  trail[13] = new RealPoint(8.0, 5.0);
	  trail[14] = new RealPoint(9.0, 5.0);
	  trail[15] = new RealPoint(10.0, 5.0);
	  trail[16] = new RealPoint(11.0, 5.0);
	  trail[17] = new RealPoint(12.0, 5.0);
	  trail[18] = new RealPoint(12.0, 6.0);
	  trail[19] = new RealPoint(12.0, 7.0);
	  trail[20] = new RealPoint(12.0, 8.0);
	  trail[21] = new RealPoint(12.0, 9.0);
	  trail[22] = new RealPoint(12.0, 10.0);
	  trail[23] = new RealPoint(12.0, 11.0);
	  trail[24] = new RealPoint(12.0, 12.0);
	  trail[25] = new RealPoint(12.0, 13.0);
	  trail[26] = new RealPoint(12.0, 14.0);
	  trail[27] = new RealPoint(12.0, 15.0);
	  trail[28] = new RealPoint(12.0, 16.0);
	  trail[29] = new RealPoint(12.0, 17.0);
	  trail[30] = new RealPoint(12.0, 18.0);
	  trail[31] = new RealPoint(12.0, 19.0);
	  trail[32] = new RealPoint(12.0, 20.0);
	  trail[33] = new RealPoint(12.0, 21.0);
	  trail[34] = new RealPoint(12.0, 22.0);
	  trail[35] = new RealPoint(12.0, 23.0);
	  trail[36] = new RealPoint(12.0, 24.0);
	  trail[37] = new RealPoint(11.0, 24.0);
	  trail[38] = new RealPoint(10.0, 24.0);
	  trail[39] = new RealPoint(9.0, 24.0);
	  trail[40] = new RealPoint(8.0, 24.0);
	  trail[41] = new RealPoint(7.0, 24.0);
	  trail[42] = new RealPoint(6.0, 24.0);
	  trail[43] = new RealPoint(5.0, 24.0);
	  trail[44] = new RealPoint(4.0, 24.0);
	  trail[45] = new RealPoint(3.0, 24.0);
	  trail[46] = new RealPoint(2.0, 24.0);
	  trail[47] = new RealPoint(1.0, 24.0);
	  trail[48] = new RealPoint(1.0, 25.0);
	  trail[49] = new RealPoint(1.0, 26.0);
	  trail[50] = new RealPoint(1.0, 27.0);
	  trail[51] = new RealPoint(1.0, 28.0);
	  trail[52] = new RealPoint(1.0, 29.0);
	  trail[53] = new RealPoint(1.0, 30.0);
	  trail[54] = new RealPoint(2.0, 30.0);
	  trail[55] = new RealPoint(3.0, 30.0);
	  trail[56] = new RealPoint(4.0, 30.0);
	  trail[57] = new RealPoint(5.0, 30.0);
	  trail[58] = new RealPoint(6.0, 30.0);
	  trail[59] = new RealPoint(7.0, 30.0);
	  trail[60] = new RealPoint(7.0, 29.0);
	  trail[61] = new RealPoint(7.0, 28.0);
	  trail[62] = new RealPoint(7.0, 27.0);
	  trail[63] = new RealPoint(8.0, 27.0);
	  trail[64] = new RealPoint(9.0, 27.0);
	  trail[65] = new RealPoint(10.0, 27.0);
	  trail[66] = new RealPoint(11.0, 27.0);
	  trail[67] = new RealPoint(12.0, 27.0);
	  trail[68] = new RealPoint(13.0, 27.0);
	  trail[69] = new RealPoint(14.0, 27.0);
	  trail[70] = new RealPoint(15.0, 27.0);
	  trail[71] = new RealPoint(16.0, 27.0);
	  trail[72] = new RealPoint(16.0, 26.0);
	  trail[73] = new RealPoint(16.0, 25.0);
	  trail[74] = new RealPoint(16.0, 24.0);
	  trail[75] = new RealPoint(16.0, 23.0);
	  trail[76] = new RealPoint(16.0, 22.0);
	  trail[77] = new RealPoint(16.0, 21.0);
	  trail[78] = new RealPoint(16.0, 20.0);
	  trail[79] = new RealPoint(16.0, 19.0);
	  trail[80] = new RealPoint(16.0, 18.0);
	  trail[81] = new RealPoint(16.0, 17.0);
	  trail[82] = new RealPoint(16.0, 16.0);
	  trail[83] = new RealPoint(16.0, 15.0);
	  trail[84] = new RealPoint(17.0, 15.0);
	  trail[85] = new RealPoint(18.0, 15.0);
	  trail[86] = new RealPoint(19.0, 15.0);
	  trail[87] = new RealPoint(20.0, 15.0);
	  trail[88] = new RealPoint(20.0, 14.0);
	  trail[89] = new RealPoint(20.0, 13.0);
	  trail[90] = new RealPoint(20.0, 12.0);
	  trail[91] = new RealPoint(20.0, 11.0);
	  trail[92] = new RealPoint(20.0, 10.0);
	  trail[93] = new RealPoint(20.0, 9.0);
	  trail[94] = new RealPoint(20.0, 8.0);
	  trail[95] = new RealPoint(20.0, 7.0);
	  trail[96] = new RealPoint(20.0, 6.0);
	  trail[97] = new RealPoint(20.0, 5.0);
	  trail[98] = new RealPoint(21.0, 5.0);
	  trail[99] = new RealPoint(22.0, 5.0);
	  trail[100] = new RealPoint(23.0, 5.0);
	  trail[101] = new RealPoint(24.0, 5.0);
	  trail[102] = new RealPoint(24.0, 4.0);
	  trail[103] = new RealPoint(24.0, 3.0);
	  trail[104] = new RealPoint(24.0, 2.0);
	  trail[105] = new RealPoint(25.0, 2.0);
	  trail[106] = new RealPoint(26.0, 2.0);
	  trail[107] = new RealPoint(27.0, 2.0);
	  trail[108] = new RealPoint(28.0, 2.0);
	  trail[109] = new RealPoint(29.0, 2.0);
	  trail[110] = new RealPoint(29.0, 3.0);
	  trail[111] = new RealPoint(29.0, 4.0);
	  trail[112] = new RealPoint(29.0, 5.0);
	  trail[113] = new RealPoint(29.0, 6.0); 
	  trail[114] = new RealPoint(29.0, 7.0);
	  trail[115] = new RealPoint(29.0, 8.0);
	  trail[116] = new RealPoint(29.0, 9.0);
	  trail[117] = new RealPoint(29.0, 10.0);
	  trail[118] = new RealPoint(29.0, 11.0);
	  trail[119] = new RealPoint(29.0, 12.0);
	  trail[120] = new RealPoint(29.0, 13.0);
	  trail[121] = new RealPoint(29.0, 14.0);
	  trail[122] = new RealPoint(28.0, 14.0);
	  trail[123] = new RealPoint(27.0, 14.0);
	  trail[124] = new RealPoint(26.0, 14.0);
	  trail[125] = new RealPoint(25.0, 14.0);
	  trail[126] = new RealPoint(24.0, 14.0);
	  trail[127] = new RealPoint(23.0, 14.0);
	  trail[128] = new RealPoint(23.0, 15.0);
	  trail[129] = new RealPoint(23.0, 16.0); 
	  trail[130] = new RealPoint(23.0, 17.0);
	  trail[131] = new RealPoint(23.0, 18.0);
	  trail[132] = new RealPoint(24.0, 18.0);
	  trail[133] = new RealPoint(25.0, 18.0);
	  trail[134] = new RealPoint(26.0, 18.0);
	  trail[135] = new RealPoint(27.0, 18.0);
	  trail[136] = new RealPoint(27.0, 19.0);
	  trail[137] = new RealPoint(27.0, 20.0);
	  trail[138] = new RealPoint(27.0, 21.0);
	  trail[139] = new RealPoint(27.0, 22.0);
	  trail[140] = new RealPoint(26.0, 22.0);
	  trail[141] = new RealPoint(25.0, 22.0);
	  trail[142] = new RealPoint(24.0, 22.0);
	  trail[143] = new RealPoint(23.0, 22.0);
	  trail[144] = new RealPoint(23.0, 23.0);

	  return trail;
}

}





